@@ -22,6 +22,7 @@
 # SOFTWARE.
 
 import gdb
+from .lib import *
 
 
 def read_reg(inf, addr, len):
@@ -29,11 +30,6 @@ def read_reg(inf, addr, len):
     return sum(v << (8*i) for i, v in enumerate(bs))
 
 
-def format_int(val, bits):
-    chars = (bits + 3) // 4
-    return (("0" * chars) + ("%x" % (val,)))[-chars:]
-
-
 class ArgType:
     def __init__(self, name, completer=None, getter=None, optional=False):
         self.name = name
@@ -137,7 +133,7 @@ class RegisterDef:
         self.size = size
         self.fields = fields
 
-    def dump(self, inf, include_descr=True):
+    def dump(self, inf, include_descr=True, base=4, all=False):
         m_int = read_reg(inf, self.addr, self.size)
 
         if self.descr and include_descr:
@@ -145,71 +141,74 @@ class RegisterDef:
         else:
             descr = ""
 
-        print("%-32s = %8s %s" %
-              (self.name, format_int(m_int, self.size * 8), descr)
+        field_align = "%%%ds" % (32//base)
+        print(("%-32s = "+field_align+" %s") %
+              (self.name, format_int(m_int, self.size * 8, base=base), descr)
               )
 
         for field in self.fields:
-            if field.should_print(m_int):
-                field.print(m_int, include_descr)
+            if all or field.should_print(m_int):
+                field.print(m_int, include_descr, base=base)
 
 
 class Field:
-    def __init__(self, name, descr=None):
+    def __init__(self, name, descr=None, always=False):
         self.name = name
         self.descr = descr
+        self.always = always
 
     def should_print(self, value):
         return False
 
-    def get_bitmask(self, value):
+    def get_value(self, value):
         return 0
 
+    def get_print_bits(self, value, base=4):
+        return format_int(value, 32, 0, 0, base)
+
     def get_print_value(self, value):
         return None
 
-    def print(self, value, include_descr=True):
-        bitmask = self.get_bitmask(value)
-        print_value = self.get_print_value(value)
-
+    def print(self, value, include_descr=True, base=4):
         if self.descr and include_descr:
             descr = (" // " + self.descr)
         else:
             descr = ""
 
+        field_align = "%%%ds" % (32//base)
         print(
-            "    %-28s   %8s - %-15s%s" %
+            ("    %-28s   "+field_align+" - %-15s%s") %
             (
                 self.name,
-                format_int(bitmask, 32),
-                print_value,
+                self.get_print_bits(value, base=base),
+                self.get_print_value(value),
                 descr
             )
         )
 
 
 class FieldBitfield(Field):
-    def __init__(self, name, bit_offset, bit_width, descr=None):
-        super().__init__(name, descr)
+    def __init__(self, name, bit_offset, bit_width, descr=None, always=False):
+        super().__init__(name, descr, always=always)
         self.bit_offset = bit_offset
         self.bit_width = bit_width
 
-    def get_bitmask(self, value):
-        return value & (((1 << self.bit_width)-1) << self.bit_offset)
+    def should_print(self, value):
+        return self.always or self.get_value(value) != 0
 
     def get_value(self, value):
         return (value >> self.bit_offset) & ((1 << self.bit_width)-1)
 
+    def get_print_bits(self, value, base=4):
+        return format_int(value, 32, self.bit_offset, self.bit_width, base)
+
     def get_print_value(self, value):
         return format_int(self.get_value(value), self.bit_width)
 
-    def should_print(self, value):
-        return self.get_value(value) != 0
-
 
 class FieldBitfieldEnum(FieldBitfield):
-    def __init__(self, name, bit_offset, bit_width, enum_values, descr=None):
-        super().__init__(name, bit_offset, bit_width, descr)
+    def __init__(self, name, bit_offset, bit_width, enum_values, descr=None, always=False):
+        super().__init__(name, bit_offset, bit_width, descr, always=always)
         self.enum_values = enum_values
 
     def get_enum_value(self, value):
@@ -227,6 +226,8 @@ class FieldBitfieldEnum(FieldBitfield):
             return format_int(self.get_value(value), self.bit_width)
 
     def should_print(self, value):
+        if self.always:
+            return True
         try:
             v, is_default, name, descr = self.get_enum_value(value)
             return not is_default
@@ -235,6 +236,6 @@ class FieldBitfieldEnum(FieldBitfield):
 
 
 class FieldBit(FieldBitfield):
-    def __init__(self, name, bit, descr=None):
-        super().__init__(name, bit, 1, descr)
+    def __init__(self, name, bit, descr=None, always=False):
+        super().__init__(name, bit, 1, descr, always=always)
         self.bit = bit
