@@ -29,9 +29,11 @@ from .common import *
 class ArmToolsSysTick (ArgCommand):
     """Dump of ARM Cortex-M SysTick block
 
-Usage: arm systick [/h]
+Usage: arm systick [/hab]
 
 Modifier /h provides descriptions of names where available
+Modifier /a Print all fields, including default values
+Modifier /b prints bitmasks in binary instead of hex
 """
 
     # https://developer.arm.com/documentation/dui0552/a/cortex-m3-peripherals/system-timer--systick
@@ -58,6 +60,8 @@ Modifier /h provides descriptions of names where available
     def __init__(self):
         super().__init__('arm systick', gdb.COMMAND_DATA)
         self.add_mod('h', 'descr')
+        self.add_mod('a', 'all')
+        self.add_mod('b', 'binary')
 
     def invoke(self, argument, from_tty):
         args = self.process_args(argument)
@@ -65,6 +69,8 @@ Modifier /h provides descriptions of names where available
             self.print_help()
             return
 
+        base = 1 if args['binary'] else 4
+
         inf = gdb.selected_inferior()
         for reg in self.regs:
-            reg.dump(inf, args['descr'])
+            reg.dump(inf, args['descr'], base=base, all=args['all'])
